

!TODO CHECK


!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   qa_analyze_mod
!> @brief   run analyzing trajectories
!! @authors Norio Takase (NT)
!
!  (c) Copyright 2014 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

#ifdef HAVE_CONFIG_H
#include "../../../config.h"
#endif

module qa_analyze_mod

  use qa_option_str_mod
  use fileio_trj_mod
  use measure_mod
  use trajectory_str_mod
  use output_str_mod
  use molecules_str_mod
  use fileio_mod
  use messages_mod
  use constants_mod
 
  implicit none
  private

  ! subroutines
  public  :: analyze
  private :: analyze_qnt
  private :: out_result

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    analyze
  !> @brief        run analyzing trajectories
  !! @authors      NT
  !! @param[in]    trj_list   : trajectory file list information
  !! @param[in]    output     : output information
  !! @param[inout] option     : option information
  !! @param[inout] trajectory : trajectory information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine analyze(molecule, trj_list, output, option, trajectory)

    ! formal arguments
    type(s_molecule),        intent(in)    :: molecule
    type(s_trj_list),        intent(in)    :: trj_list
    type(s_output),          intent(in)    :: output
    type(s_option),          intent(inout) :: option
    type(s_trajectory),      intent(inout) :: trajectory

    ! local variables
    type(s_trj_file)         :: trj_in
    integer                  :: nstru, ifile, istep, num_trjfiles, qnt_unit


    if (option%check_only) &
      return

    ! open output file
    !
    call open_file(qnt_unit, output%qntfile, IOFileOutputNew)


    ! analysis loop
    !
    nstru = 0
    num_trjfiles = size(trj_list%md_steps)

    do ifile = 1, num_trjfiles

      ! open trajectory file
      !
      call open_trj(trj_in, trj_list%filenames(ifile), &
                            trj_list%trj_format,       &
                            trj_list%trj_type, IOFileInput)

      do istep = 1, trj_list%md_steps(ifile)

        ! read trajectory
        !   coordinates of one MD snapshot are saved in trajectory%coord)
        !
        call read_trj(trj_in, trajectory)

        if (mod(istep, trj_list%ana_periods(ifile)) == 0) then

          nstru = nstru + 1
          write(MsgOut,*) '      number of structures = ', nstru

          call analyze_qnt(trajectory, option)
          call out_result (nstru, qnt_unit, option%qnative)

        end if

      end do

      ! close trajectory file
      !
      call close_trj(trj_in)

    end do


    ! close output file
    !
    call close_file(qnt_unit)

    return

  end subroutine analyze

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    analyze_qnt
  !> @brief        analysis qnative
  !! @authors      NT
  !! @param[in]    trajectory : trajectory informatin
  !! @param[inout] option     : option information
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine analyze_qnt(trajectory, option)

    ! formal argments
    type(s_trajectory),      intent(in)    :: trajectory
    type(s_option),          intent(inout) :: option

    ! local variables
    real(wp)                 :: kai, r
    integer                  :: i1, i2, nfound
    integer                  :: ncon, nspa
    integer                  :: icon, ispa


    ncon = size(option%contact_list(1,:))

    nfound = 0

    ! find native contact about whole system and, selection pairs
    !
    do icon = 1, ncon

      i1  = option%contact_list(1,icon)
      i2  = option%contact_list(1,icon)
      kai = option%contact_kai   (icon)

      r   = compute_dis(trajectory%coord(:,i1), &
                        trajectory%coord(:,i2))

      if (r > kai) &
        cycle

      nfound = nfound + 1
      ispa = option%contact_sel_pair(icon)

      if (ispa /= 0) &
        option%sel_pair_nfound(ispa) = option%sel_pair_nfound(ispa) + 1

    end do


    ! setup qnative value
    !
    option%qnative(1) = &
                        real(nfound, wp) / &
                        real(ncon, wp)
    do ispa = 1, nspa
      option%qnative(ispa+1) = &
                        real(option%sel_pair_nfound(ispa), wp) / &
                        real(option%sel_pair_ncontact(ispa), wp)
    end do

    return

  end subroutine analyze_qnt

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine    out_result
  !> @brief        output analysis results
  !! @authors      NT
  !! @param[in]    sturct_no : sturcture number
  !! @param[in]    unit_no   : file unit number
  !! @param[in]    results   : analysis results
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine out_result(struct_no, unit_no, results)

    ! formal arguments
    integer,                 intent(in)    :: struct_no
    integer,                 intent(in)    :: unit_no
    real(wp),                intent(in)    :: results(:)

    ! local variables
    integer                  :: i


    write(unit_no, '(i10,1x,100(f8.3,1x))') &
         struct_no, (results(i), i=1, size(results))

    return

  end subroutine out_result

end module qa_analyze_mod
